///////////////////////////////////////////////////////////////////////////////////////////////////
// I. LICENSE CONDITIONS
//
// Copyright (c) 2018 by Disney-Pixar
//
// Permission is hereby granted to use this software solely for non-commercial applications 
// and purposes including academic or industrial research, evaluation and not-for-profit media
// production. All other rights are retained by Pixar. For use for or in connection with 
// commercial applications and purposes, including without limitation in or in connection with 
// software products offered for sale or for-profit media production, please contact Pixar at 
// tech-licensing@pixar.com.
//
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
// NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS 
// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL PIXAR OR ITS AFFILIATES BE 
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
// IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
///////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef KELVINLET_DYNA_PULSE_BASE_H
#define KELVINLET_DYNA_PULSE_BASE_H

#include "kelvinlet/dynaBase.h"

///
/// \class DynaPulseBase
/// \brief Base class for a Pulse Dynamic Kelvinlet.
///
/// It corresponds to the displacement generated by an impulse load.
/// The Pseudo-Potential can be written as the difference of the
/// W-potential and its derivative.
///

namespace Kelvinlet 
{

class DynaPulseBase : public DynaBase
{
protected:
    //////////////////////
    // Override Methods //
    //////////////////////

    virtual Scalar _EvalU(const Scalar r, const Scalar t, const Scalar a) const override;
    virtual Scalar _EvalGradU(const Scalar r, const Scalar t, const Scalar a) const override;
    virtual Scalar _EvalHessU(const Scalar r, const Scalar t, const Scalar a) const override;

    ///////////////////////
    // Impulse Potential //
    ///////////////////////

    Scalar _EvalU0(const Scalar t, const Scalar a) const;
    Scalar _EvalW(const Scalar r, const Scalar R) const;
    Scalar _EvalGradW(const Scalar r, const Scalar R) const;
    Scalar _EvalHessW(const Scalar r, const Scalar R) const;
};

} // namespace Kelvinlet 

#endif // KELVINLET_DYNA_PULSE_BASE_H
